VG_ECME <- function(y, mu, sigma2, gammap, nu, Delta, eps, maxiter)
{

  # This function estimates the parameters of the Variance-gamma distirbution using the ECME algorithm; see
  # Bee, M., Dickson, M.M. and Santi, F. (2017), "Likelihood-based Risk Estimation for Variance-Gamma Models",
  # DEM working paper 2017/03, http://www.economia.unitn.it/71/dem-working-papers. 
  
  # Inputs:  
  # y: numeric vector of data;
  # mu: starting value of mu
  # sigma2: starting value of sigma^2;
  # gamma: starting value of gamma;
  # nu: starting value of nu;
  # Delta: value of Delta to be used n the E step;
  # eps: tolerance
  # maxiter: maximum number of iterations.
  
  # Output:
  
  # parsECME: (6x1) numeric vector; the forst 4 vaues are  the estimates of mu, sigma^2, gamma and nu,
  # the fifth value is the optimized log-likelihood and the last value is the number of iterations.
  
  # Usage: 
  
  # library(tseries)
  # library(zoo)
  # library(lsr)
  # source('VG_ECME.R')
  # price<- get.hist.quote(instrument = "^GSPC",start = "2008-01-02",end = "2008-12-31", quote="AdjClose")
  # r <- diff(log(price))
  # dem_r <- r - mean(r)
  # y <- coredata(dem_r)
  # y <- as.vector(y)
  # nus = 2
  # gammas = 0
  # Delta = 1.e-2
  # sigmas = aad(y) # mean absolute deviation
  # mus = mean(y)
  # eps <- 10^(-8)                                   # convergence criterion
  # maxiter = 2000
  # parsECME = VG_ECME(y, mus, sigmas^2, gammas, nus, Delta, eps, maxiter)
  
source('lVG.r')
N <- length(y)                                     # number of observations
change <- 1                           # initial test value for convergence
llik = rep(-10^10,(maxiter+1))
nit <- 0                                    # initialize iteration counter
param <- c(mu, sigma2, gammap, nu) # arrange all parameters in a vector
Sy = sum(y)
# h = 10^(-5)
while (change > eps && nit <= maxiter)             # start iterations
{
  options(digits = 6)	    # format for the display of numerical results
  parold <- param                               # store old parameter values
  changeold <- change
  delta = sqrt((y - mu)^2/sigma2)
  delta1 = Delta/sqrt(2*nu+gammap^2/sigma2)
  
  # E-step 1
  
  lambda = (delta * besselK(sqrt(2*nu+gammap^2/sigma2)*delta,nu+.5, expon.scaled = T)) / (sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta,nu-.5, expon.scaled = T))
  lambdainv = rep(0,length(lambda))
  indici1 = which(delta>=Delta/sqrt(2*nu+gammap^2/sigma2))
  indici2 = which(delta<Delta/sqrt(2*nu+gammap^2/sigma2))
  lambdainv[indici1] = sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta[indici1],nu-1.5, expon.scaled = T) / (delta[indici1] * besselK(sqrt(2*nu+gammap^2/sigma2)*delta[indici1],nu-.5, expon.scaled = T))
  lambdainv[indici2] = sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta1,nu-1.5, expon.scaled = T) / (delta1 * besselK(sqrt(2*nu+gammap^2/sigma2)*delta1,nu-.5, expon.scaled = T))
  Sylambda = sum(y*lambdainv)
  Slambda = sum(lambda)
  S1lambda = sum(lambdainv)
  
  # CM-step 1
  
  mu = (Sylambda*Slambda - N*Sy) / (S1lambda*Slambda - N^2)
  gammap = (Sy - N*mu) / Slambda
  lambda = (delta * besselK(sqrt(2*nu+gammap^2/sigma2)*delta,nu+.5, expon.scaled = T)) / (sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta,nu-.5, expon.scaled = T))
  Slambda = sum(lambda)
  lambdainv = rep(0,length(lambda))
  lambdainv[indici1] = sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta[indici1],nu-1.5, expon.scaled = T) / (delta[indici1] * besselK(sqrt(2*nu+gammap^2/sigma2)*delta[indici1],nu-.5, expon.scaled = T))
  lambdainv[indici2] = sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta1,nu-1.5, expon.scaled = T) / (delta1 * besselK(sqrt(2*nu+gammap^2/sigma2)*delta1,nu-.5, expon.scaled = T))
  sigma2 = sum((y-mu)^2*lambdainv)/N - gammap^2*Slambda/N
  
  # E-step 2
  
  lambda = (delta * besselK(sqrt(2*nu+gammap^2/sigma2)*delta,nu+.5, expon.scaled = T)) / (sqrt(2*nu+gammap^2/sigma2) * besselK(sqrt(2*nu+gammap^2/sigma2)*delta,nu-.5, expon.scaled = T))
  Slambda = sum(lambda)

  # CM-step 2

  results1 = optimize(lVG, c(0,20), tol = 1e-10, maximum = TRUE, mu, sigma2, gammap, y) 
  nu = results1$maximum
  param <- c(mu, sigma2, gammap, nu) # arrange all parameters in a vector
  change <- max(abs(param - parold))           # test value for convergence
  print(c(nit,change,param)) 
  nit <- nit + 1                               # increase iteration counter
}
c(param,llik[nit+1],(nit+1))
}
